﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GpsModule {
	public class GgaStrategy {

		public delegate void updateErrorEventHandler(Exception error);
		public event updateErrorEventHandler updateErrorEvent;

		GgaData _data;
		public GgaData Data {
			get { return _data; }
		}

		public GgaStrategy() {
			_data = new GgaData();
		}

		public void parseSentance(string sentence) {
			try {
				_data.setSentance(sentence);
				//1) Time (UTC)
				sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
				string s = sentence.Substring(0, sentence.IndexOf(","));
				int hour;
				int.TryParse(s.Substring(0, 2), out hour);
				int minute;
				int.TryParse(s.Substring(2, 2), out minute);
				int second;
				int.TryParse(s.Substring(4, 2), out second);
				//2) Latitude
				sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
				_data.setLatitude(sentence.Substring(0, sentence.IndexOf(",")));
				//3) N or S (North or South)
				sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
				_data.setNorthOrSouth(sentence.Substring(0, sentence.IndexOf(","))[0]);
				//4) Longitude
				sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
				_data.setLongitude((sentence.Substring(0, sentence.IndexOf(","))));
				//5) E or W (East or West)
				sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
				_data.setEastOrWest(sentence.Substring(0, sentence.IndexOf(","))[0]);
				//6) GPS Quality Indicator, 0 - fix not available, 1 - GPS fix, 2 - Differential GPS fix
				sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
				_data.setQualityIndicator(int.Parse(sentence.Substring(0, sentence.IndexOf(","))));
				//if fix not available dont do anything else
				if (_data.QualityIndicator > 0) {
					//7) Number of satellites in view, 00 - 12
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					_data.setSatelitesInView(int.Parse(sentence.Substring(0, sentence.IndexOf(","))));
					//8) Horizontal Dilution of precision
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					_data.setHorizontalDilutionOfPrecision(double.Parse(sentence.Substring(0, sentence.IndexOf(","))));
					//9) Antenna Altitude above/below mean-sea-level (geoid)
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					_data.setAntennaAltitude(double.Parse(sentence.Substring(0, sentence.IndexOf(","))));
					//10) Units of antenna altitude, meters
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					_data.setUnitsOfGeoidalSeperation(sentence.Substring(0, sentence.IndexOf(","))[0]);
					//11) Geoidal separation, the difference between the WGS-84 
					//earth ellipsoid and mean-sea-level (geoid), "-" means mean-sea-level below ellipsoid
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					_data.setGeoidalSeparation(double.Parse(sentence.Substring(0, sentence.IndexOf(","))));
					//12) Units of geoidal separation, meters
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					_data.setUnitsOfAntennaAltitude(sentence.Substring(0, sentence.IndexOf(","))[0]);
					//13) Age of differential GPS data, time in seconds since last SC104
					//type 1 or 9 update, null field when DGPS is not used
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					//_data.setAgeOfDifferentialGPSData(double.Parse(sentence.Substring(0, sentence.IndexOf(","))));
					//14) Differential reference station ID, 0000-1023
					sentence = sentence.Substring(sentence.IndexOf(",") + 1, sentence.Length - sentence.IndexOf(",") - 1);
					//_data.setDifferentialReferenceStationID(int.Parse(sentence.Substring(0, sentence.IndexOf("*"))));
					//15) Checksum
					sentence = sentence.Substring(sentence.IndexOf("*") + 1, sentence.Length - sentence.IndexOf("*") - 1);
					//_checksum = sentence.Substring(0, sentence.Length);
				}
				//set the time
				_data.setTime(new DateTime(DateTime.Now.Year, DateTime.Now.Month
					, DateTime.Now.Day, hour, minute, second, DateTimeKind.Utc));
			} catch (Exception e) {
				if (updateErrorEvent != null)
					updateErrorEvent.Invoke(e);
			}
		}
	}
}
