﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RS232Port;

namespace GpsModule {
	public class GpsStrategy : SerialStrategy {

		RmcStrategy _rmc;
		public RmcStrategy Rmc {
			get { return _rmc; }
		}

		GgaStrategy _gga;
		public GgaStrategy Gga {
			get { return _gga; }
		}

		GpsData _data;
		public GpsData Data {
			get { return _data; }
		}

		public GpsStrategy(string portName) {
			serialPortName = portName;
			_data = new GpsData();
			_rmc = new RmcStrategy();
			_rmc.updateErrorEvent += updateError;
			_gga = new GgaStrategy();
			_gga.updateErrorEvent += updateError;
			openSerialPortEvent += initializeGpsModulePort;
			initializeGpsModulePort();
		}

		private void initializeGpsModulePort() {
			_rs232 = new RS232MainForm();
			_rs232.updateErrorEvent += updateError;
			_rs232.Port.PortName = serialPortName;
			_rs232.Show();
			_rs232.Hide();
			_rs232.Port.BaudRate = 38400;
			_rs232.receiveDataEvent += receiveData;
			_rs232.setSilentModeEvent += updateSilentMode;
			_rs232.updatePortSettings();
			_rs232.Text = "GPS Module Serial Port";
			_rs232.DebugMode = RS232MainForm.debugTerminalMode.silent;
			_rs232.Port.ReceivedBytesThreshold = 600;
			_status = _rs232.initializeComPort();

		}

		private void receiveData() {
			_data.setBuffer(_rs232.SerialData);
			string s = _data.Buffer;
			try {
				while (s.Contains("$GP") && s.Contains("\r\n")
					&& s.IndexOf("$GP") < s.LastIndexOf("\r\n")) {
					string sentence = s.Substring(s.IndexOf("$GP") + 3
						, s.IndexOf("\r\n", s.IndexOf("$GP")) - (s.IndexOf("$GP") + 3));
					s = s.Remove(0, s.IndexOf("\r\n") + 2);
					//computeChecksum(sentence);
					if (sentence.Contains("GGA"))
						_gga.parseSentance(sentence);
					else if (sentence.Contains("RMC") && _gga.Data.QualityIndicator > 0)
						_rmc.parseSentance(sentence);
				}
			} catch (Exception e) {
				updateError(e);
				_data.flushBuffer();
			}
			updateData();
			updateUi();
		}

		private int computeChecksum(string data) {
			//int checksum = 0;
			byte testByte;
			foreach (char c in data) {
				testByte = (byte)c;
			}
			//checksum += ((int)(byte)c);

			return 0;
		}
	}
}
